import argparse
import glob
import os
from typing import Optional, Union

import soundfile as sf
from rich.progress import track


def __get_files(dir_name: str, extensions: Union[list[str], set[str]]) -> set[str]:
    """Helper function to get files in a single directory"""

    # Expand out the directory
    dir_name = os.path.abspath(os.path.expanduser(dir_name))

    files = set()

    for sub_ext in extensions:
        globstr = os.path.join(dir_name, "*" + os.path.extsep + sub_ext)
        files |= set(glob.glob(globstr))

    return files


def find_files(
    directory: str,
    ext: Optional[Union[list, set]] = None,
    recurse: Optional[bool] = True,
    case_sensitive: Optional[bool] = False,
) -> list[str]:
    """
    Find files in a directory.

    Args:
        directory (str): The directory to search.
        ext (str): The extension to search for.
        recurse (bool): Whether to recurse into subdirectories.
        case_sensitive (bool): Whether to do a case sensitive search.
    """
    if ext is None:
        ext = ["aac", "au", "flac", "m4a", "mp3", "ogg", "wav"]

    elif isinstance(ext, str):
        ext = [ext]

    # Cast into a set
    ext = set(ext)

    # Generate upper-case versions
    if not case_sensitive:
        # Force to lower-case
        ext = set([e.lower() for e in ext])
        # Add in upper-case versions
        ext |= set([e.upper() for e in ext])

    files = set()

    if recurse:
        for walk in os.walk(directory):
            files |= __get_files(walk[0], ext)
    else:
        files = __get_files(directory, ext)

    files = list(files)
    files.sort()
    return files


def duration_str(duration) -> str:
    hours, rest = divmod(duration, 3600)
    minutes, seconds = divmod(rest, 60)
    if hours >= 1:
        duration = f"{hours:.0f}h {minutes:.0f}min {seconds:.3f}s"
    elif minutes >= 1:
        duration = "{0:02.0g}:{1:05.3f} min".format(minutes, seconds)
    else:
        duration = f"{seconds:.2f}s"
    return duration


def main() -> None:
    parser = argparse.ArgumentParser(
        prog="audioinfo", description="A tool to check audio file durations in a directory."
    )
    parser.add_argument(
        "--directory",
        "-d",
        help="The directory to search",
    )
    parser.add_argument(
        "--ext",
        "-e",
        nargs="+",
        default="wav",
        help="The extension to search for",
    )
    parser.add_argument(
        "--recurse",
        "-r",
        default=True,
        action="store_true",
        help="Recurse into subdirectories",
    )
    parser.add_argument(
        "--case-sensitive",
        "-c",
        default=False,
        action="store_true",
        help="Case sensitive search",
    )
    args = parser.parse_args()

    print("Searching for files...")
    files = find_files(args.directory, args.ext, args.recurse, args.case_sensitive)
    print(f"Finished searching for {len(files)} files.")

    if len(files) == 0:
        print(f"No files found in {args.directory}, please check your arguments.")
        return

    total_duration = 0.0
    for file in track(files, description="Analyzing files:"):
        with sf.SoundFile(file) as f:
            name = f.name
            samplerate = f.samplerate
            channels = f.channels
            frames = f.frames
            duration = float(frames) / f.samplerate
            format = f.format

            print(
                f"{name} - {samplerate} - {format} - # channels: {channels}, duration: {duration:.3f}s"
            )

            total_duration += duration

    print(f"Total number of files: {len(files)}.")
    print(f"Total duration: {duration_str(total_duration)}.")
    print(f"Average duration: {duration_str(total_duration / len(files))}.")



import importlib_metadata

# -- Project information -----------------------------------------------------
project = "audioinfo"
author = "HAO Xiang <haoxiangsnr@gmail.com>"
project_copyright = "2022, HAO Xiang"
release = importlib_metadata.version(project)
version = ".".join(
    release.split(".")[:2]
)  # e.g., "0.3" stand for the major is "0" and the minor is "3"

# -- MetaConfig configuration ---------------------------------------------------
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
extensions = [
    "myst_parser",  # markdown file parser.
    "sphinx.ext.todo",  # enable the todo.
    "sphinx.ext.autodoc",  # provide automatic documentation for module (*.py), class, function, and  typehints.
    "sphinx.ext.autosummary",  # auto-generate the summary (include links) of the modules.
    "sphinx.ext.intersphinx",  # enable cross-referencing between Sphinx projects.
    "sphinx.ext.viewcode",  # add a helpful link to the source code of each object in the API reference sheet.
    "sphinx.ext.mathjax",  # enable math support in the documentation.
    "sphinx.ext.napoleon",  # [ordered] parse our docstrings and generate Google-style docstrings.
    "sphinxcontrib.autodoc_pydantic",  # generate the suitable docstrings to pydantic models.
]

# -- Extension configuration -------------------------------------------------
napoleon_numpy_docstring = False
napoleon_attr_annotations = True
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "numpy": ("https://numpy.org/doc/stable/", None),
}
autosummary_generate = True
autodoc_mock_imports = ["soundfile", "gpuRIR"]
autodoc_pydantic_model_signature_prefix = "Config"
autodoc_pydantic_member_order = "bysource"
autodoc_pydantic_model_show_field_summary = False
autodoc_pydantic_model_show_json = False
autodoc_pydantic_model_show_validator_members = False
autodoc_pydantic_model_show_validator_summary = False
autodoc_pydantic_model_summary_list_order = "bysource"
autodoc_pydantic_model_list_validators = False
autodoc_pydantic_field_signature_prefix = "option"

# -- Options for HTML output -------------------------------------------------
html_theme = "sphinx_rtd_theme"
html_context = {
    "display_github": True,  # edit on Github, see https://github.com/readthedocs/sphinx_rtd_theme/issues/529
    "github_user": "haoxiangsnr",
    "github_repo": "audioinfo",
    "github_version": "main",
}
html_static_path = ["_static"]
html_css_files = [
    "css/custom.css",
]
